package de.meinefirma.main;

import java.util.Arrays;

public class VergleichJaxbVersusJaxbMitStax
{
   static final String ENCODING = "UTF-8";

   public static void main( String[] args ) throws Exception
   {
      if( args.length < 3 ) {
         System.out.println( "\nBitte Klasse, XSD-Schema und XML-Dokument(e) angeben, z.B.:\n" +
                             "java de.meinefirma.main.VergleichJaxbVersusJaxbMitStax" +
                             " de.meinefirma.buecherentities.Buecher BuecherOhneKeyref.xsd Buecher10.xml Buecher100.xml" );
         return;
      }
      vergleiche( args[0], args[1], Arrays.copyOfRange( args, 2, args.length ) );
   }

   public static void vergleiche( String clazz, String xsdSchema, String[] xmlDateien ) throws Exception
   {
      Class<?> clss        = Class.forName( clazz );
      String   packageName = clss.getPackage().getName();
      System.out.println( "\nSchema: " + xsdSchema + ", Klasse: " + clazz );
      Thread.sleep( 3000 );

      // JIT vorbereiten:
      XmlParserMitStaxUndJaxb.parseXmlElemente(
            xsdSchema, xmlDateien[0], ENCODING, packageName, new XmlParserMitStaxUndJaxbTest.DefaultElementeVerarbeitungImpl() );
      XmlParserMitStaxUndJaxb.parseXmlElemente(
            xsdSchema, xmlDateien[0], ENCODING, packageName, new XmlParserMitStaxUndJaxbTest.DefaultElementeVerarbeitungImpl() );

      // JAXB mit StAX, ohne Validierung:
      System.out.println( "\nJAXB mit StAX, ohne Validierung:" );
      for( String xmlDatei : xmlDateien ) {
         testeJaxbMitStax( null, xmlDatei, packageName );
      }
      // JAXB mit StAX, mit Validierung:
      System.out.println( "\nJAXB mit StAX, mit Validierung:" );
      for( String xmlDatei : xmlDateien ) {
         testeJaxbMitStax( xsdSchema, xmlDatei, packageName );
      }

      // JIT vorbereiten:
      JaxbMarshalUnmarshalUtil.unmarshal( xsdSchema, xmlDateien[0], clss );
      JaxbMarshalUnmarshalUtil.unmarshal( xsdSchema, xmlDateien[0], clss );

      // JAXB ohne StAX, ohne Validierung:
      System.out.println( "\nJAXB ohne StAX, ohne Validierung:" );
      for( String xmlDatei : xmlDateien ) {
         testeNurJaxb( null, xmlDatei, clss );
      }
      // JAXB ohne StAX, mit Validierung:
      System.out.println( "\nJAXB ohne StAX, mit Validierung:" );
      for( String xmlDatei : xmlDateien ) {
         testeNurJaxb( xsdSchema, xmlDatei, clss );
      }

      System.gc();
   }

   static void testeJaxbMitStax( String xsdSchema, String xmlDatei, String packageName ) throws Exception
   {
      System.out.println( "XML-Datei = " + xmlDatei + ":" );
      long startZeit = System.nanoTime();
      long anzahlElemente = XmlParserMitStaxUndJaxb.parseXmlElemente(
            xsdSchema, xmlDatei, ENCODING, packageName, new XmlParserMitStaxUndJaxbTest.DefaultElementeVerarbeitungImpl() );
      String dauer = JaxbMarshalUnmarshalUtil.ermittleDauer( startZeit );
      System.out.println( "Elementeanzahl = " + anzahlElemente + ", Parsingdauer = " + dauer );
   }

   static void testeNurJaxb( String xsdSchema, String xmlDatei, Class<?> clss ) throws Exception
   {
      System.out.println( "XML-Datei = " + xmlDatei + ":" );
      long startSpeicherverbrauch = JaxbMarshalUnmarshalUtil.ermittleSpeicherverbrauch();
      long startZeit = System.nanoTime();
      Object obj = JaxbMarshalUnmarshalUtil.unmarshal( xsdSchema, xmlDatei, clss );
      String dauer = JaxbMarshalUnmarshalUtil.ermittleDauer( startZeit );
      String speicherverbrauch = JaxbMarshalUnmarshalUtil.formatiereSpeichergroesse(
            JaxbMarshalUnmarshalUtil.ermittleSpeicherverbrauch() - startSpeicherverbrauch );
      if( !(obj.getClass().equals(clss)) ) throw new Exception( "Falsche Klasse." );
      System.out.println( "Parsingspeicherverbrauch = " + speicherverbrauch + ", Parsingdauer = " + dauer );
   }
}
