package de.meinefirma.main;

import java.io.*;
import java.util.Iterator;
import javax.xml.bind.*;
import javax.xml.stream.*;
import de.meinefirma.buecherentities.*;

public class XmlErzeugerMitStaxUndJaxb
{
   public static void main( String[] args ) throws Exception
   {
      String outFile = ( args.length > 0 ) ? args[0] : null;
      String charEnc = ( args.length > 1 ) ? args[1] : "UTF-8";
      erzeugeXML( outFile, charEnc );
      if( outFile != null ) {
         try( Reader rd = new InputStreamReader( new FileInputStream( outFile ) ) ) {
            XmlParserMitStaxUndJaxb.validate( "Buecher.xsd", rd );
         }
      }
   }

   public static void erzeugeXML( String outXmlFile, String xmlCharEnc ) throws Exception
   {
      try( OutputStream os = ( outXmlFile == null ) ? System.out : new FileOutputStream( new File( outXmlFile ) );
           Writer wr = new OutputStreamWriter( os, xmlCharEnc ) ) {
         XMLStreamWriter xsw = XMLOutputFactory.newInstance().createXMLStreamWriter( wr );
         // xsw = new IndentingXMLStreamWriter( xsw );
         try {
            Marshaller marshaller = JAXBContext.newInstance( Buecher.class ).createMarshaller();
            marshaller.setProperty( Marshaller.JAXB_ENCODING, xmlCharEnc );
            marshaller.setProperty( Marshaller.JAXB_FRAGMENT, Boolean.TRUE );

            xsw.writeStartDocument( xmlCharEnc, null );
            xsw.writeCharacters( "\r\n" );
            xsw.writeStartElement( "buecher" );
            xsw.setDefaultNamespace(   "http://meinnamespace.meinefirma.de" );
            xsw.writeDefaultNamespace( "http://meinnamespace.meinefirma.de" );
            xsw.writeNamespace( "xsi", "http://www.w3.org/2001/XMLSchema-instance" );
            xsw.writeAttribute( "xsi:schemaLocation", "http://meinnamespace.meinefirma.de Buecher.xsd" );
            xsw.writeCharacters( "\r\n" );

            AutorenDatenLieferant a = new AutorenDatenLieferant();
            VerlageDatenLieferant v = new VerlageDatenLieferant();
            BuecherDatenLieferant b = new BuecherDatenLieferant();
            while( a.hasNext() ) { marshaller.marshal( a.next(), xsw ); xsw.writeCharacters( "\r\n" ); }
            while( v.hasNext() ) { marshaller.marshal( v.next(), xsw ); xsw.writeCharacters( "\r\n" ); }
            while( b.hasNext() ) { marshaller.marshal( b.next(), xsw ); xsw.writeCharacters( "\r\n" ); }

            xsw.writeEndElement();
            xsw.writeEndDocument();
            xsw.writeCharacters( "\r\n" );
            xsw.flush();
         } finally {
            xsw.close();
         }
      }
   }

   static class AutorenDatenLieferant implements Iterator<AutorType> {
      private int idx = 0;
      @Override public void remove() { throw new UnsupportedOperationException(); }
      @Override public boolean hasNext() { return idx < 1; }
      @Override public AutorType next() {
         AutorType a = new AutorType();
         a.setId( 40 + ++idx );
         a.setName( "Otto" );
         a.setOrt( "Hamburg" );
         return a;
      }
   }

   static class VerlageDatenLieferant implements Iterator<VerlagType> {
      private int idx = 0;
      @Override public void remove() { throw new UnsupportedOperationException(); }
      @Override public boolean hasNext() { return idx < 1; }
      @Override public VerlagType next() {
         VerlagType v = new VerlagType();
         v.setId( 4000 + ++idx );
         v.setName( "Elefanten-Verlag" );
         v.setOrt( "Berlin" );
         return v;
      }
   }

   static class BuecherDatenLieferant implements Iterator<BuchType> {
      private int idx = 0;
      @Override public void remove() { throw new UnsupportedOperationException(); }
      @Override public boolean hasNext() { return idx < 2; }
      @Override public BuchType next() {
         BuchType b = new BuchType();
         b.setAutorID( 41 );
         b.setVerlagID( 4001 );
         b.setTitel( "Meine Elefanten " + ++idx );
         return b;
      }
   }
}
